;;; -*- Mode: Lisp; Package: System -*-
;;;
;;; **********************************************************************
;;; This code was written as part of the CMU Common Lisp project at
;;; Carnegie Mellon University, and has been placed in the public domain.
;;; If you want to use this code or any part of CMU Common Lisp, please contact
;;; Scott Fahlman or slisp-group@cs.cmu.edu.
;;;
(ext:file-comment
  "$Header: config.lisp,v 1.4 93/07/26 15:16:03 ram Exp $")
;;;
;;; **********************************************************************
;;;
;;; Utility to load subsystems and save a new core.
;;;
(in-package "USER")


(block abort
  (let ((output-file #-linux #p"library:lisp.core"
                     #+linux #p"home:lisp.core")
	(load-clm #-linux t #+linux nil)
	(load-clx t)
	(load-clue t)
	(load-clio t)
	(load-pictures t)
	(load-hemlock t)
	(load-lisp-tk t)
	(load-defsystem t)
	(other ()))
    (loop
      (fresh-line)
      (format t " 1: specify result file (currently ~S)~%"
	      (namestring output-file))
      (format t " 2: toggle loading of the CLX X library, currently ~
		 ~:[dis~;en~]abled.~%"
	      load-clx)
      (format t " 3: toggle loading of the Clue library, currently ~
		 ~:[dis~;en~]abled. ~
                 ~:[~%    (would force loading of CLX)~;~]~%"
	      load-clue load-clx)
      (format t " 4: toggle loading of the Clio library, currently ~
		 ~:[dis~;en~]abled. ~
                 ~:[~%    (would force loading of Clue)~;~] ~
                 ~:[~%    (would force loading of CLX)~;~]~%"
	      load-clio load-clue load-clx)
      (format t " 5: toggle loading of the Pictures library, currently ~
                 ~:[dis~;en~]abled. ~
                 ~:[~%    (would force loading of Clue)~;~] ~
                 ~:[~%    (would force loading of CLX)~;~]~%"
              load-pictures load-clue load-clx)
      (format t " 6: toggle loading of Motif and the graphical debugger, ~
		 currently ~:[dis~;en~]abled.~
		 ~:[~%    (would force loading of CLX.)~;~]~%"
	      load-clm load-clx)
      (format t " 7: toggle loading the Hemlock editor, currently ~
		 ~:[dis~;en~]abled.~
		 ~:[~%    (would force loading of CLX.)~;~]~%"
	      load-hemlock load-clx)
      (format t " 8: toggle loading of the Lisp-Tk library, currently ~
		 ~:[dis~;en~]abled.~%"
	      load-lisp-tk)
      (format t " 9: toggle loading of the defsystem library, currently ~
		 ~:[dis~;en~]abled.~%"
	      load-defsystem)
      (format t " 90: specify some site-specific file to load.~@
		 ~@[    Current files:~%~{      ~S~%~}~]"
	      (mapcar #'namestring other))
      (format t " 99: configure according to current options.~%")
      (format t " 0: abort the configuration process.~%")
      (format t "~%Option number: ")
      (force-output)
      (flet ((file-prompt (prompt)
	       (format t prompt)
	       (force-output)
	       (pathname (string-trim " 	" (read-line)))))
	(let ((res (ignore-errors (read-from-string (read-line)))))
	  (case res
	    (1
	     (setq output-file (file-prompt "Result core file name: ")))
	    (2
	     (unless (setq load-clx (not load-clx))
	       (progn
		 (setq load-hemlock nil)
                 (setq load-pictures nil)
		 (setq load-clio nil)
		 (setq load-clue nil))))
	    (3
	     (if (setq load-clue (not load-clue))
               (setq load-clx t)
	       (progn
		 (setq load-clio nil)
                 (setq load-pictures nil))))
	    (4
	     (setq load-clio (not load-clio))
             (when load-clio
	      (setq load-clue t)
              (setq load-clx t)))
            (5
             (setq load-pictures (not load-pictures))
             (when load-pictures
              (setq load-clue t)
              (setq load-clx t)))
	    (6
	     (when (setq load-clm (not load-clm))
	       (setq load-clx t)))
	    (7
	     (when (setq load-hemlock (not load-hemlock))
	       (setq load-clx t)))
	    (8 (setq load-lisp-tk (not load-lisp-tk)))
	    (9 (setq load-defsystem (not load-defsystem)))
	    (90
	     (setq other
		   (append other
			   (list (file-prompt "File(s) to load ~
					       (can have wildcards): ")))))

	    (99 (return))
	    (0
	     (format t "~%Aborted.~%")
	     (return-from abort))))))
    
    (gc-off)
    (when load-clx
      (setf *features* (delete :no-clx *features* :test #'eq))
      (load "library:subsystems/clx-library"))
    (when load-clue
      (setf *features* (delete :no-clue *features* :test #'eq))
      (setf (getf ext:*herald-items* :clue)
        `("    Clue Library 1"))
      (load "library:subsystems/clue-library"))
    (when load-clio
      (setf *features* (delete :no-clio *features* :test #'eq))
      (pushnew :clio *features*)
      (setf (getf ext:*herald-items* :clio)
        `("    Clio Library 1"))
      (load "library:subsystems/clio-library"))
    (when load-pictures
      (setf *features* (delete :no-pictures *features* :test #'eq))
      (pushnew :pictures *features*)
      (setf (getf ext:*herald-items* :pictures)
        `("    Pictures Library 1"))
      (load "library:subsystems/pictures-library"))
    (when load-clm
      (setf *features* (delete :no-clm *features* :test #'eq))
      (load "library:subsystems/clm-library"))
    (when load-hemlock
      (setf *features* (delete :no-hemlock *features* :test #'eq))
      (load "library:subsystems/hemlock-library"))
    (when load-lisp-tk
      (setf *features* (delete :no-lisp-tk *features* :test #'eq))
      (setf (getf ext:*herald-items* :clue)
	    `("    Lisp-Tk 0.1"))
      (load "library:subsystems/lisp-tk"))

    (when load-defsystem
      (setf (getf ext:*herald-items* :defsystem)
	    `("    Defsystem Mar 13 1995"))
      (load "library:subsystems/defsystem-library"))
      
    (dolist (f other) (load f))
    
    (setq *info-environment*
	  (list* (make-info-environment :name "Working")
		 (compact-info-environment (first *info-environment*)
					   :name "Auxiliary")
		 (rest *info-environment*)))
    
    (when (probe-file output-file)
      (multiple-value-bind
	  (ignore old new)
	  (rename-file output-file
		       (concatenate 'string (namestring output-file)
				    ".BAK"))
	(declare (ignore ignore))
	(format t "~&Saved ~S as ~S.~%" (namestring old) (namestring new))))
    
    ;;
    ;; Enable the garbage collector.  But first fake it into thinking that
    ;; we don't need to garbage collect.  The save-lisp is going to call
    ;; purify so any garbage will be collected then.
    (setf lisp::*need-to-collect-garbage* nil)
    (gc-on)
    ;;
    ;; Save the lisp.
    (save-lisp output-file)))

(quit)
